-------------------------------------------------------------------------------
--
--  Copyright (c) 2010-2018 Dave Newbold , University of Bristol/ STFC RAL
--
--  This source file may be used and distributed without restriction provided
--  that this copyright statement is not removed from the file and that any
--  derivative work contains the original copyright notice and the associated
--  disclaimer.
--
--  This source file is free software: you can redistribute it and/or modify it
--  under the terms of the GNU Lesser General Public License as published by
--  the Free Software Foundation, either version 3 of the License, or (at your
--  option) any later version.
--
--  This source file is distributed in the hope that it will be useful, but
--  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
--  or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
--  for more details.
--
--  You should have received a copy of the GNU Lesser General Public License
--  along with the SoLiD/PUMA firmware.  If not, see http://www.gnu.org/licenses
--
-------------------------------------------------------------------------------
-- freq_ctr_div
--
-- General clock frequency monitor
--
-- This part divides a local clock by 64, the output to be sent to the
-- frequency counter part.
--
-- Dave Newbold, September 2013

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
library unisim;
use unisim.VComponents.all;

entity freq_ctr_div is
	generic(
		N_CLK: positive := 1
	);
	port(
		clk: in std_logic_vector(N_CLK - 1 downto 0);
		clkdiv: out std_logic_vector(N_CLK - 1 downto 0)
	);
	
end freq_ctr_div;

architecture rtl of freq_ctr_div is

	signal t: std_logic_vector(N_CLK - 1 downto 0) := (others => '0');

begin

	div_gen: for i in N_CLK - 1 downto 0 generate
	
		signal q: std_logic;
		signal cnt: natural range 0 to 31;
		
	begin
		
		process(clk(i))
		begin
			if rising_edge(clk(i)) then
				
				if cnt = 0 then
					cnt <= 31;
					q <= '1';
				else
					cnt <= cnt - 1;
					q <= '0';					
				end if;
					
				if q = '1' then
					t(i) <= not t(i);
				end if;
				
			end if;
		end process;
		
	end generate;
	
	clkdiv <= t;
	
end rtl;
