-------------------------------------------------------------------------------
--
--  Copyright (c) 2010-2018 Dave Newbold , University of Bristol/ STFC RAL
--
--  This source file may be used and distributed without restriction provided
--  that this copyright statement is not removed from the file and that any
--  derivative work contains the original copyright notice and the associated
--  disclaimer.
--
--  This source file is free software: you can redistribute it and/or modify it
--  under the terms of the GNU Lesser General Public License as published by
--  the Free Software Foundation, either version 3 of the License, or (at your
--  option) any later version.
--
--  This source file is distributed in the hope that it will be useful, but
--  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
--  or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
--  for more details.
--
--  You should have received a copy of the GNU Lesser General Public License
--  along with the SoLiD/PUMA firmware.  If not, see http://www.gnu.org/licenses
--
-------------------------------------------------------------------------------
-- Address decode logic for ipbus fabric
-- 
-- This file has been AUTOGENERATED from the address table - do not hand edit
-- 
-- We assume the synthesis tool is clever enough to recognise exclusive conditions
-- in the if statement.
-- 
-- Dave Newbold, February 2011

library IEEE;
use IEEE.STD_LOGIC_1164.all;
use ieee.numeric_std.all;

package ipbus_decode_mp7_ttc is

  constant IPBUS_SEL_WIDTH: positive := 5; -- Should be enough for now?
  subtype ipbus_sel_t is std_logic_vector(IPBUS_SEL_WIDTH - 1 downto 0);
  function ipbus_sel_mp7_ttc(addr : in std_logic_vector(31 downto 0)) return ipbus_sel_t;

-- START automatically  generated VHDL the Mon Oct 19 15:30:59 2015 
  constant N_SLV_CSR: integer := 0;
  constant N_SLV_FREQ: integer := 1;
  constant N_SLV_L1_GEN: integer := 2;
  constant N_SLV_HIST: integer := 3;
  constant N_SLV_CMD_CTRS: integer := 4;
  constant N_SLV_TMT: integer := 5;
  constant N_SLAVES: integer := 6;
-- END automatically generated VHDL

    
end ipbus_decode_mp7_ttc;

package body ipbus_decode_mp7_ttc is

  function ipbus_sel_mp7_ttc(addr : in std_logic_vector(31 downto 0)) return ipbus_sel_t is
    variable sel: ipbus_sel_t;
  begin

-- START automatically  generated VHDL the Mon Oct 19 15:30:59 2015 
    if    std_match(addr, "---------------------------00---") then
      sel := ipbus_sel_t(to_unsigned(N_SLV_CSR, IPBUS_SEL_WIDTH)); -- csr / base 0x00000000 / mask 0x00000018
    elsif std_match(addr, "---------------------------0100-") then
      sel := ipbus_sel_t(to_unsigned(N_SLV_FREQ, IPBUS_SEL_WIDTH)); -- freq / base 0x00000008 / mask 0x0000001e
    elsif std_match(addr, "---------------------------0101-") then
      sel := ipbus_sel_t(to_unsigned(N_SLV_L1_GEN, IPBUS_SEL_WIDTH)); -- l1_gen / base 0x0000000a / mask 0x0000001e
    elsif std_match(addr, "---------------------------011--") then
      sel := ipbus_sel_t(to_unsigned(N_SLV_HIST, IPBUS_SEL_WIDTH)); -- hist / base 0x0000000c / mask 0x0000001c
    elsif std_match(addr, "---------------------------10---") then
      sel := ipbus_sel_t(to_unsigned(N_SLV_CMD_CTRS, IPBUS_SEL_WIDTH)); -- cmd_ctrs / base 0x00000010 / mask 0x00000018
    elsif std_match(addr, "---------------------------1100-") then
      sel := ipbus_sel_t(to_unsigned(N_SLV_TMT, IPBUS_SEL_WIDTH)); -- tmt / base 0x00000018 / mask 0x0000001e
-- END automatically generated VHDL

    else
        sel := ipbus_sel_t(to_unsigned(N_SLAVES, IPBUS_SEL_WIDTH));
    end if;

    return sel;

  end function ipbus_sel_mp7_ttc;

end ipbus_decode_mp7_ttc;

