-------------------------------------------------------------------------------
--
--  Copyright (c) 2010-2018 Dave Newbold , University of Bristol/ STFC RAL
--
--  This source file may be used and distributed without restriction provided
--  that this copyright statement is not removed from the file and that any
--  derivative work contains the original copyright notice and the associated
--  disclaimer.
--
--  This source file is free software: you can redistribute it and/or modify it
--  under the terms of the GNU Lesser General Public License as published by
--  the Free Software Foundation, either version 3 of the License, or (at your
--  option) any later version.
--
--  This source file is distributed in the hope that it will be useful, but
--  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
--  or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
--  for more details.
--
--  You should have received a copy of the GNU Lesser General Public License
--  along with the SoLiD/PUMA firmware.  If not, see http://www.gnu.org/licenses
--
-------------------------------------------------------------------------------
-- ttc_del
--
-- Fine delay element for incoming TTC data
--
-- Dave Newbold, July 2013

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
library unisim;
use unisim.VComponents.all;

entity ttc_del is
	port(
		i: in std_logic;
		o: out std_logic;
		refclk: in std_logic;
		rst: in std_logic;
		rdy: out std_logic;
		clk: in std_logic;
		inc: in std_logic
	);

end ttc_del;

architecture rtl of ttc_del is

	attribute IODELAY_GROUP: string;
	attribute IODELAY_GROUP of ctrl: label is "ttc_del";
	attribute IODELAY_GROUP of idel: label is "ttc_del";

	begin

	ctrl: IDELAYCTRL
		port map(
			refclk => refclk,
			rst => rst,
			rdy => rdy
		);
		
	idel: IODELAYE1
		generic map(IDELAY_TYPE => "VARIABLE")
		port map(
			c => clk,
			t => '0',
			rst => rst,
			ce => inc,
			inc => '1',
			cinvctrl => '0',
			cntvaluein => "00000",
			clkin => '0',
			idatain => i,
			datain => '0',
			odatain => '0',
			dataout => o,
			cntvalueout => open
		);

end rtl;
