-------------------------------------------------------------------------------
--
--  Copyright (c) 2010-2018 Dave Newbold , University of Bristol/ STFC RAL
--
--  This source file may be used and distributed without restriction provided
--  that this copyright statement is not removed from the file and that any
--  derivative work contains the original copyright notice and the associated
--  disclaimer.
--
--  This source file is free software: you can redistribute it and/or modify it
--  under the terms of the GNU Lesser General Public License as published by
--  the Free Software Foundation, either version 3 of the License, or (at your
--  option) any later version.
--
--  This source file is distributed in the hope that it will be useful, but
--  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
--  or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
--  for more details.
--
--  You should have received a copy of the GNU Lesser General Public License
--  along with the SoLiD/PUMA firmware.  If not, see http://www.gnu.org/licenses
--
-------------------------------------------------------------------------------
----------------------------------------------------------------------------------
-- Lukas Arnold, University of Bristol
-- 21 September 2016
-- SoLid Experiment
-- r0.01
----------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use ieee.numeric_std.all;

use work.ipbus.all;
use work.ipbus_reg_types.all;

use work.top_decl.all;


entity sc_npeaks_thresh is

generic( VAL_WIDTH:	natural := 14 -- bit
);

port(	rst:		in std_logic;
	clk:		in std_logic;
	req:		in std_logic; -- request information
	d:		in std_logic_vector(VAL_WIDTH-1 DOWNTO 0);
	threshold_trig:	in std_logic_vector(VAL_WIDTH-1 DOWNTO 0);
	threshold_fe:	in std_logic_vector(VAL_WIDTH-1 DOWNTO 0);
	trig:		out std_logic
);
end sc_npeaks_thresh;

architecture rtl of sc_npeaks_thresh is
    signal feat_val : std_logic_vector(VAL_WIDTH-1 DOWNTO 0);
begin

	featextract: entity work.sc_npeaks
	generic map (
		windowlength => 14, -- bits
		VAL_WIDTH => VAL_WIDTH -- bits
	)
	port map(
		rst => rst,
		clk => clk,
		data_in => d,
		threshold => threshold_fe,
		npeaks => feat_val
	);

	thresh : entity work.sc_thresh
	generic map (
		VAL_WIDTH	=> VAL_WIDTH -- bits
	)
	port map(
		rst 		=> rst,
		clk		=> clk,
		req		=> req,
		val		=> feat_val,
		threshold	=> threshold_trig,
		trig		=> trig
	);

end architecture rtl;
