-------------------------------------------------------------------------------
--
--  Copyright (c) 2010-2018 Dave Newbold , University of Bristol/ STFC RAL
--
--  This source file may be used and distributed without restriction provided
--  that this copyright statement is not removed from the file and that any
--  derivative work contains the original copyright notice and the associated
--  disclaimer.
--
--  This source file is free software: you can redistribute it and/or modify it
--  under the terms of the GNU Lesser General Public License as published by
--  the Free Software Foundation, either version 3 of the License, or (at your
--  option) any later version.
--
--  This source file is distributed in the hope that it will be useful, but
--  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
--  or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
--  for more details.
--
--  You should have received a copy of the GNU Lesser General Public License
--  along with the SoLiD/PUMA firmware.  If not, see http://www.gnu.org/licenses
--
-------------------------------------------------------------------------------
-- scaled_ctr
--
-- Exponent + mantissa counter for large values
--
-- Dave Newbold, August 2016

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use ieee.numeric_std.all;

entity scaled_ctr is
	generic(
		MANTISSA_BITS: natural := 8;
		EXPONENT_BITS: natural := 8
	);
	port(
		clk: in std_logic;
		rst: in std_logic;
		inc: in std_logic;
		m: out std_logic_vector(MANTISSA_BITS - 1 downto 0);
		e: out std_logic_vector(EXPONENT_BITS - 1 downto 0)		
	);

end scaled_ctr;

architecture rtl of scaled_ctr is

	signal mctr: unsigned(MANTISSA_BITS - 1 downto 0);
	signal ectr: unsigned(EXPONENT_BITS - 1 downto 0);

begin

	process(clk)
	begin
		if rising_edge(clk) then
			if rst = '1' then
				mctr <= (others => '0');
				ectr <= (others => '0');
			elsif inc = '1' then
				if mctr = (mctr'range => '1') then
					if ectr /= (ectr'range => '1') then
						mctr(mctr'left) <= '1';
						mctr(mctr'left - 1 downto 0) <= (others => '0');
						ectr <= ectr + 1;
					end if;
				else
					mctr <= mctr + 1;
				end if;
			end if;
		end if;
	end process;

	m <= std_logic_vector(mctr);
	e <= std_logic_vector(ectr);
	
end rtl;
