-------------------------------------------------------------------------------
--
--  Copyright (c) 2010-2018 Dave Newbold , University of Bristol/ STFC RAL
--
--  This source file may be used and distributed without restriction provided
--  that this copyright statement is not removed from the file and that any
--  derivative work contains the original copyright notice and the associated
--  disclaimer.
--
--  This source file is free software: you can redistribute it and/or modify it
--  under the terms of the GNU Lesser General Public License as published by
--  the Free Software Foundation, either version 3 of the License, or (at your
--  option) any later version.
--
--  This source file is distributed in the hope that it will be useful, but
--  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
--  or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
--  for more details.
--
--  You should have received a copy of the GNU Lesser General Public License
--  along with the SoLiD/PUMA firmware.  If not, see http://www.gnu.org/licenses
--
-------------------------------------------------------------------------------
-- sync_routing
--
-- Switchyard for the various ways of using the HDMI connector
--
-- sync_a input is always routed to clk_pll, but might be unused at PLL end
-- sync_a output is always crappy clk40
-- sync_b is an output for ctrl = '0', but an input for ctrl = '1'
-- sync_c exists on the cable but is not used
--
-- Dave Newbold, August 2016

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

library unisim;
use unisim.VComponents.all;

entity sync_routing is
	port(
		clk40: in std_logic;
		ctrl: in std_logic;
		sync_out: out std_logic;
		sync_in: in std_logic;
		sync_a_p: inout std_logic;
		sync_a_n: inout std_logic;
		sync_b_p: inout std_logic;
		sync_b_n: inout std_logic;
		clk_pll_p: out std_logic;
		clk_pll_n: out std_logic
	);

end sync_routing;

architecture rtl of sync_routing is

	signal sync_a_i, sync_a_o: std_logic;

begin

	oclk: ODDR
		port map(
			q => sync_a_o,
			c => clk40,
			ce => '1',
			d1 => '0',
			d2 => '1',
			r => '0',
			s => '0'
		); -- DDR register for clock forwarding

	ia: IOBUFDS
		port map(
			O => sync_a_i,
			IO => sync_a_p,
			IOB => sync_a_n,
			I => sync_a_o,
			T => '0'
		);
					
	obclk: OBUFDS
		port map(
			i => sync_a_i,
			o => clk_pll_p,
			ob => clk_pll_n
		);
			
	ib: IOBUFDS
		port map(
			O => sync_out,
			IO => sync_b_p,
			IOB => sync_b_n,
			I => sync_in,
			T => ctrl
		);
		
end rtl;
