-------------------------------------------------------------------------------
--
--  Copyright (c) 2010-2018 Dave Newbold , University of Bristol/ STFC RAL
--
--  This source file may be used and distributed without restriction provided
--  that this copyright statement is not removed from the file and that any
--  derivative work contains the original copyright notice and the associated
--  disclaimer.
--
--  This source file is free software: you can redistribute it and/or modify it
--  under the terms of the GNU Lesser General Public License as published by
--  the Free Software Foundation, either version 3 of the License, or (at your
--  option) any later version.
--
--  This source file is distributed in the hope that it will be useful, but
--  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
--  or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
--  for more details.
--
--  You should have received a copy of the GNU Lesser General Public License
--  along with the SoLiD/PUMA firmware.  If not, see http://www.gnu.org/licenses
--
-------------------------------------------------------------------------------
-- Address decode logic for ipbus fabric
-- 
-- This file has been AUTOGENERATED from the address table - do not hand edit
-- 
-- We assume the synthesis tool is clever enough to recognise exclusive conditions
-- in the if statement.
-- 
-- Dave Newbold, February 2011

library IEEE;
use IEEE.STD_LOGIC_1164.all;
use ieee.numeric_std.all;

package ipbus_decode_sc_io is

  constant IPBUS_SEL_WIDTH: positive := 5; -- Should be enough for now?
  subtype ipbus_sel_t is std_logic_vector(IPBUS_SEL_WIDTH - 1 downto 0);
  function ipbus_sel_sc_io(addr : in std_logic_vector(31 downto 0)) return ipbus_sel_t;

-- START automatically  generated VHDL the Wed Feb 17 16:06:58 2016 
  constant N_SLV_CSR: integer := 0;
  constant N_SLV_FREQ_CTR: integer := 1;
  constant N_SLV_CLOCK_I2C: integer := 2;
  constant N_SLV_SPI: integer := 3;
  constant N_SLV_ANALOG_I2C: integer := 4;
  constant N_SLAVES: integer := 5;
-- END automatically generated VHDL

    
end ipbus_decode_sc_io;

package body ipbus_decode_sc_io is

  function ipbus_sel_sc_io(addr : in std_logic_vector(31 downto 0)) return ipbus_sel_t is
    variable sel: ipbus_sel_t;
  begin

-- START automatically  generated VHDL the Wed Feb 17 16:06:58 2016 
    if    std_match(addr, "---------------------------000--") then
      sel := ipbus_sel_t(to_unsigned(N_SLV_CSR, IPBUS_SEL_WIDTH)); -- csr / base 0x00000000 / mask 0x0000001c
    elsif std_match(addr, "---------------------------001--") then
      sel := ipbus_sel_t(to_unsigned(N_SLV_FREQ_CTR, IPBUS_SEL_WIDTH)); -- freq_ctr / base 0x00000004 / mask 0x0000001c
    elsif std_match(addr, "---------------------------01---") then
      sel := ipbus_sel_t(to_unsigned(N_SLV_CLOCK_I2C, IPBUS_SEL_WIDTH)); -- clock_i2c / base 0x00000008 / mask 0x00000018
    elsif std_match(addr, "---------------------------10---") then
      sel := ipbus_sel_t(to_unsigned(N_SLV_SPI, IPBUS_SEL_WIDTH)); -- spi / base 0x00000010 / mask 0x00000018
    elsif std_match(addr, "---------------------------11---") then
      sel := ipbus_sel_t(to_unsigned(N_SLV_ANALOG_I2C, IPBUS_SEL_WIDTH)); -- analog_i2c / base 0x00000018 / mask 0x00000018
-- END automatically generated VHDL

    else
        sel := ipbus_sel_t(to_unsigned(N_SLAVES, IPBUS_SEL_WIDTH));
    end if;

    return sel;

  end function ipbus_sel_sc_io;

end ipbus_decode_sc_io;

